Modules
- mpsc
- Multi-producer, single-consumer channels.
Structs
- Barrier
- 🧵
stdEnables multiple threads to synchronize the beginning of some computation. - Barrier
Wait Result - 🧵
⚖️
stdReturned byBarrier::wait()when all threads in it have rendezvoused. - Condvar
- 🧵
stdA Condition Variable. - Lazy
Lock - 🧵
🌱
stdA value which is initialized on the first access. - Mutex
- 🧵
stdA mutual exclusion primitive useful for protecting shared data. - Mutex
Guard - 🧵
🔒
stdAn RAII implementation of a “scoped lock” of a mutex. - Once
- 🧵
stdA synchronization primitive for one-time global initialization. - Once
Lock - 🧵
stdA synchronization primitive which can be written to only once. - Once
State - 🧵
stdState yielded toOnce::call_once_force()’s closure parameter. - Poison
Error - 🧵
🚩
stdA type of error which can be returned whenever a lock is acquired. - RwLock
- 🧵
stdA reader-writer lock - RwLock
Read Guard - 🧵
stdRAII structure used to release the shared read access of a lock when dropped. - RwLock
Write Guard - 🧵
stdRAII structure used to release the exclusive write access of a lock when dropped. - Wait
Timeout Result - 🧵
⚖️
stdWhether a timed wait on aCondvarreturned due to a time out or not.
Enums
- TryLock
Error - 🧵
🚩+
stdAn enumeration of possible errors associated with aTryLockResult.
Type Aliases
- Lock
Result - 🧵
⚖️
stdA type alias for the result of a lock method which can be poisoned. - TryLock
Result - 🧵
⚖️
stdA type alias for the result of a nonblocking locking method.